@@ -132,6 +132,7 @@ static void FindPointParents(struct FemType *data,struct BoundaryType *bound,
   for(elemind=1;elemind<=data->noelements;elemind++) {
     elemtype = data->elementtypes[elemind];
     elemsides = elemtype % 100;
+
     for(i=0;i<elemsides;i++) {
       elemhits[data->topology[elemind][i]] += 1;
     }
@@ -256,6 +257,7 @@ static void FindPointParents(struct FemType *data,struct BoundaryType *bound,
       }
     }    
   }
+
   
   free_Ivector(indx,1,data->noknots);
 
@@ -349,7 +351,7 @@ int LoadAbaqusInput(struct FemType *data,struct BoundaryType *bound,
    */
 {
   int noknots,noelements,elemcode,maxnodes,material;
-  int mode,allocated,nvalue,maxknot,nosides;
+  int mode,allocated,nvalue,nvalue2,maxknot,nosides;
   int boundarytype,boundarynodes;
   int *nodeindx,*boundindx;
   
@@ -392,10 +394,11 @@ omstart:
   material = 0;
   ivalues0[0] = ivalues0[1] = 0;
 
+
   for(;;) {
     /* getline; */
 
-    if (Getrow(line,in,FALSE)) goto end;
+    if (Getrow(line,in,TRUE)) goto end;
 
     /* if(!line) goto end; */
     /* if(strstr(line,"END")) goto end; */
@@ -424,6 +427,8 @@ omstart:
 	  elemcode = 408;
 	else if(strstr(line,"3D8"))
 	  elemcode = 808;
+	else if(strstr(line,"3D20"))
+	  elemcode = 820;
 	else if(strstr(line,"3D4"))
 	  elemcode = 504;
 	else printf("Unknown element code: %s\n",line);
@@ -472,12 +477,24 @@ omstart:
 	
       case 3:
 	noelements++;
+
+	nvalue = StringToInteger(line,ivalues,MAXNODESD2+1,',');
+	
 	if(allocated) {
-	  nvalue = StringToInteger(line,ivalues,MAXNODESD2+1,',');
 	  data->elementtypes[noelements] = elemcode;
-	  for(i=1;i<nvalue;i++)
-	    data->topology[noelements][i-1] = ivalues[i];
 	  data->material[noelements] = material;
+	  for(i=0;i<nvalue-1;i++) 
+	    data->topology[noelements][i] = ivalues[i+1];	  
+	}
+	
+	if(nvalue < elemcode % 100) {
+	  Getrow(line,in,TRUE);
+	  if(allocated) {
+	    if(ivalues[nvalue-1] == 0) nvalue--;	      
+	    nvalue2 = StringToInteger(line,ivalues,MAXNODESD2+1,',');
+	    for(i=0;i<nvalue2;i++) 
+	      data->topology[noelements][nvalue-1+i] = ivalues[i];	  	    
+	  }
 	}
 	break;
 
@@ -731,7 +748,7 @@ int LoadAbaqusOutput(struct FemType *data,char *prefix,int info)
       /* Element set */
       if(argno == 3) { 
 	elset++;
-	strcpy(data->materialname[elset],buffer);
+	strcpy(data->bodyname[elset],buffer);
       }
     case 1934:
       /* Element set continuation */
@@ -823,6 +840,184 @@ jump:
 
 
 
+int LoadNastranInput(struct FemType *data,struct BoundaryType *bound,
+		    char *prefix,int info)
+/* Load the grid from a format that in Nastran format 
+   */
+{
+  int noknots,noelements,elemcode,maxnodes,material;
+  int mode,allocated,nvalue,nvalue2,maxknot,minknot,nosides;
+  int boundarytype,boundarynodes,nodes,maxnode;
+  int *nodeindx,*boundindx;
+  
+  char filename[MAXFILESIZE];
+  char line[MAXLINESIZE],*cp;
+  int i,j,k,l,*ind;
+  FILE *in;
+  Real rvalues[MAXDOFS];
+  int ivalues[MAXDOFS],ivalues0[MAXDOFS];
+
+
+  strcpy(filename,prefix);
+  if ((in = fopen(filename,"r")) == NULL) {
+    AddExtension(prefix,filename,"nas");
+    if ((in = fopen(filename,"r")) == NULL) {
+      printf("LoadAbaqusInput: opening of the Nastran file '%s' wasn't succesfull !\n",
+	     filename);
+      return(1);
+    }
+  }
+
+  printf("Reading mesh from Nastran file %s.\n",filename);
+  InitializeKnots(data);
+
+  allocated = FALSE;
+  maxknot = 0;
+  minknot = 10000;
+
+  /* Because the file format doesn't provide the number of elements
+     or nodes the results are read twice but registered only in the
+     second time. */
+omstart:
+
+  mode = 0;
+  maxnodes = 0;
+  noknots = 0;
+  noelements = 0;
+  elemcode = 0;
+  boundarytype = 0;
+  boundarynodes = 0;
+  material = 0;
+  ivalues0[0] = ivalues0[1] = 0;
+
+
+  for(;;) {
+    /* getline; */
+
+    if (Getrow(line,in,TRUE)) goto end;
+
+    if(line[0] == '$') {
+      if(!allocated) printf("comment: %s",line);
+    }
+    else if(strstr(line,"GRID")) {
+      noknots++;
+      if(0) printf("line=%s\n",line);
+      cp = &line[5];
+      j = next_int(&cp);
+      if(0) printf("j=%d\n",j);
+
+      if(allocated) {
+	k = next_int(&cp);
+	data->x[noknots] = next_real(&cp);
+	data->y[noknots] = next_real(&cp);
+      }
+      else {
+	if(j > maxknot) maxknot = j;
+	if(j < minknot) minknot = j;
+      }
+
+      if(strstr(line,"*")) 
+	Getrow(line,in,TRUE);
+
+      if(allocated) {
+	cp = &line[4];
+	data->z[noknots] = next_real(&cp);
+      }
+
+    }
+    else if(strstr(line,"TETRA")) {
+      noelements++;
+      nodes = 4;
+      if(nodes > maxnodes) maxnodes = nodes;
+      if(allocated) {
+	data->elementtypes[noelements] = 504;
+	cp = &line[6];
+	k = next_int(&cp);
+	data->material[noelements] = next_int(&cp) + 1;	
+	for(j=0;j<nodes;j++)
+	  data->topology[noelements][j] = next_int(&cp);            
+      }      
+    }
+    else if(strstr(line,"PYRAM")) {
+      noelements++;
+      nodes = 5;
+      if(nodes > maxnodes) maxnodes = nodes;
+      if(allocated) {
+	data->elementtypes[noelements] = 605;
+	cp = &line[6];
+	k = next_int(&cp);
+	data->material[noelements] = next_int(&cp) + 1;	
+	for(j=0;j<nodes;j++)
+	  data->topology[noelements][j] = next_int(&cp);            
+      }      
+    }
+    else if(strstr(line,"PENTA")) {
+      noelements++;
+      nodes = 6;
+      if(nodes > maxnodes) maxnodes = nodes;
+      if(allocated) {
+	data->elementtypes[noelements] = 706;
+	cp = &line[6];
+	k = next_int(&cp);
+	data->material[noelements] = next_int(&cp) + 1;	
+	for(j=0;j<nodes;j++)
+	  data->topology[noelements][j] = next_int(&cp);            
+      }      
+    }
+     else if(strstr(line,"CHEXA")) {
+      noelements++;
+      nodes = 8;
+      if(nodes > maxnodes) maxnodes = nodes;
+      if(allocated) {
+	data->elementtypes[noelements] = 808;
+	cp = &line[5];
+	k = next_int(&cp);
+	data->material[noelements] = next_int(&cp) + 1;	
+	for(j=0;j<6;j++)
+	  data->topology[noelements][j] = next_int(&cp);            
+      }           
+      Getrow(line,in,TRUE);
+      if(allocated) {
+	cp = &line[1];       
+	for(j=6;j<8;j++) 
+	  data->topology[noelements][j] = k;
+      }
+    }
+    else if(strstr(line,"ENDDAT")) {
+      goto end;
+    }
+    else {
+      printf("unknown command: %s",line);
+    }
+  }
+
+  end:
+
+  if(allocated == TRUE) {
+    if(info) printf("The mesh was loaded from file %s.\n",filename);
+    fclose(in);
+    return(0);
+  }
+
+  rewind(in);
+  data->noknots = noknots;
+  data->noelements = noelements;
+  data->maxnodes = maxnodes;
+  data->dim = 3;
+  
+  printf("maxknot = %d  minknot = %d\n",maxknot,minknot);
+  
+  if(info) printf("Allocating for %d knots and %d %d-node elements.\n",
+		  noknots,noelements,maxnodes);
+  AllocateKnots(data);
+    
+  allocated = TRUE;
+  goto omstart;
+}
+
+
+
+
 static void ReorderFidapNodes(struct FemType *data,int element,int nodes,int typeflag) 
 {
   int i,j,oldtopology[MAXNODESD2],*topology,elementtype,dim;
@@ -1044,7 +1239,7 @@ int LoadFidapInput(struct FemType *data,char *prefix,int info)
   int noknots,noelements,dim,novel,elemcode,maxnodes;
   int mode,allocated,nvalue,maxknot,totelems,entity,maxentity;
   char filename[MAXFILESIZE];
-  char line[MAXLINESIZE],*cp,entityname[MAXLINESIZE],entitylist[100][MAXLINESIZE];
+  char line[MAXLINESIZE],*cp,entityname[MAXNAMESIZE],entitylist[100][MAXNAMESIZE];
   int i,j,k,*ind,geoflag,typeflag;
   int **topology;
   FILE *in;
@@ -1185,13 +1380,21 @@ int LoadFidapInput(struct FemType *data,char *prefix,int info)
 			elems,nodes,entityname);
 
 	for(entity=1;entity<=maxentity;entity++) {
+#if 0
 	  k = strcmp(entityname,entitylist[entity]);
+#else
+	  k = strcmp(entityname,data->bodyname[entity]);
+#endif
 	  if(k == 0) break;
 	}
 
 	if(entity > maxentity) {
 	  maxentity++;
+#if 0
 	  strcpy(entitylist[entity],entityname);
+#else
+	  strcpy(data->bodyname[entity],entityname);
+#endif
 	  if(info) printf("Found new entity: %s\n",entityname);
 	}
 
@@ -1277,6 +1480,9 @@ end:
 	data->topology[i][j] = ind[data->topology[i][j]];
   }
 
+
+  if(maxentity > 0) data->bodynamesexist = TRUE;
+
   fclose(in);
   
   if(info) printf("Finished reading the Fidap neutral file\n");
@@ -1387,15 +1593,13 @@ static void ReorderAnsysNodes(struct FemType *data,int *oldtopology,
   default:
     for(i=0;i<elementtype%100;i++) 
       topology[i] = oldtopology[i];
-    
-  }  
 
+  }  
 }
 
 
-
 int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
-		   char *prefix,int info)
+		      char *prefix,int info)
 /* This procedure reads the FEM mesh as written by Ansys. */
 {
   int noknots,noelements,nosides,elemcode,sidetype,currenttype;
@@ -1403,21 +1607,14 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
   int maxindx,*indx,*revindx,topology[100],ind;
   int i,j,k,l,imax,grp,dummyint,*nodeindx,*boundindx,boundarynodes,maxnodes;
   int noansystypes,*ansysdim,*ansysnodes,*ansystypes,boundarytypes;
-  int debug;
+  int debug,namesexist,maxside,sides;
   Real x,y,z,r;
   FILE *in;
   char *cp,line[MAXLINESIZE],filename[MAXFILESIZE],
-    text[MAXNAMESIZE],directoryname[MAXFILESIZE];
+    text[MAXNAMESIZE],text2[MAXNAMESIZE],directoryname[MAXFILESIZE];
 
-#if 0
-  sprintf(directoryname,"%s",prefix);
-  mkdir(directoryname,0700);
-  chdir(directoryname);
-
-  if(info) printf("Loading mesh in Ansys format from directory %s.\n",
-		  directoryname);
-#endif
 
+  /* ExportMesh.header */
 
   sprintf(filename,"%s.header",prefix);
   if ((in = fopen(filename,"r")) == NULL) {
@@ -1425,26 +1622,25 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
 	   filename);
     return(1);
   }
-  else 
-    printf("Calculating Ansys elementtypes from %s\n",filename);
 
+  if(info) printf("Calculating Ansys elementtypes from %s\n",filename);
   for(i=0;getline;i++);
-  fclose(in);
 
   noansystypes = i-1;
-  printf("LoadAnsysInput: There seems to be %d elementytypes in file %s.\n",
-	 noansystypes,filename);
+  printf("There seems to be %d elementytypes in file %s.\n",noansystypes,filename);
   
   ansysdim = Ivector(1,noansystypes);
   ansysnodes = Ivector(1,noansystypes);
   ansystypes = Ivector(1,noansystypes);
-  
-  in = fopen(filename,"r");
-  
+
+  rewind(in);
   for(i=0;i<=noansystypes;i++) {
     Real dummy1,dummy2,dummy3;
     getline;
 
+    /* Ansys writes decimal points also for integers and therefore these 
+       values are read in as real numbers. */
+
     sscanf(line,"%le %le %le",&dummy1,&dummy2,&dummy3);
 
     if(i==0) {
@@ -1463,46 +1659,34 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
   printf("Ansys file has %d elements, %d nodes and %d boundary types.\n",
 	 noelements,noknots,boundarytypes);
   
+ /* ExportMesh.names */
 
+  sprintf(filename,"%s.names",prefix);
+  in = fopen(filename,"r");
+  if(in == NULL) 
+    namesexist = FALSE;
+  else
+    namesexist = TRUE;
+
+  if(namesexist) printf("Using names of bodies and boundaries\n");
+
+
+  /* ExportMesh.node */
 
   sprintf(filename,"%s.node",prefix);
   if ((in = fopen(filename,"r")) == NULL) {
     printf("LoadAnsysInput: The opening of the nodes-file %s failed!\n",
 	   filename);
-    return(1);
-  }
-  else 
-    printf("Calculating Ansys nodes from %s\n",filename);
-
-  for(i=0;getline;i++);
-  fclose(in);
-
-  printf("LoadAnsysInput: There seems to be %d nodes in file %s.\n",
-	 i,filename);
-  if(i != noknots) printf("Conflicting number of nodes %d vs %d!\n",
-			  i,noknots);
-
-#if 0
-  sprintf(filename,"%s.elem",prefix);
-  if ((in = fopen(filename,"r")) == NULL) {
-    printf("LoadAnsysInput: The opening of the element-file %s failed!\n",
-	   filename);
     return(2);
   }
-  else 
-    printf("Calculating Ansys elements from %s\n",filename);
 
+  if(info) printf("Calculating Ansys nodes from %s\n",filename);
   for(i=0;getline;i++);
-  fclose(in);
-  
-  noelements = i;
-  if(elementtype%100 > 8) noelements /= 2;
-
-  printf("LoadAnsysInput: There seems to be %d elements in file %s.\n",
-	 noelements,filename);
-#endif
 
+  if(info) printf("There seems to be %d nodes in file %s.\n",i,filename);
+  if(i != noknots) printf("Conflicting number of nodes %d vs %d!\n",i,noknots);
 
+  /* Make room and initialize the mesh */
   InitializeKnots(data);
 
   /* Even 2D elements may form a 3D object! */
@@ -1523,23 +1707,15 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
   AllocateKnots(data);
 
   indx = Ivector(1,noknots);
+  for(i=1;i<=noknots;i++) indx[i] = 0;
 
-  for(i=1;i<=noknots;i++) 
-    indx[i] = 0;
-
-  sprintf(filename,"%s.node",prefix);
-  if ((in = fopen(filename,"r")) == NULL) {
-    printf("LoadAnsysInput: The opening of the nodes-file %s failed!\n",
-	   filename);
-    return(3);
-  }
-  else 
-    printf("Loading %d Ansys nodes from %s\n",noknots,filename);
-
+  if(info) printf("Loading %d Ansys nodes from %s\n",noknots,filename);
+  rewind(in);
   for(i=1;i<=noknots;i++) {
     getline; cp=line;
 
     indx[i] = next_int(&cp);
+    if(cp[0] == '.') cp++;
 
     x = next_real(&cp);
     if(!cp) x = y = z = 0.0;
@@ -1554,21 +1730,8 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
     data->x[i] = x;
     data->y[i] = y;
     if(data->dim == 3) data->z[i] = z;
-
-#if 0
-    if(i != indx[i]) {
-      printf("Indexes differ: i=%d indx=%d\n",i,indx[i]);
-      printf("line: %s",line);
-    }
-    if(0 && data->dim==3) printf("i:%d  ind:%d  x:%.3lg  y:%.3lg  z:%.3lg\n",
-	   i,indx[i],data->x[i],data->y[i],data->z[i]);
-    if(0 && data->dim==2) printf("i:%d  ind:%d  x:%.3lg  y:%.3lg\n",
-	   i,indx[i],data->x[i],data->y[i]);
-#endif
   }
   fclose(in);
-
-
   
   /* reorder the indexes */
   maxindx = indx[1];
@@ -1589,6 +1752,7 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
       revindx[i] = i;
   }
 
+  /* ExportMesh.elem */
 
   sprintf(filename,"%s.elem",prefix);
   if ((in = fopen(filename,"r")) == NULL) {
@@ -1596,12 +1760,13 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
 	   filename);
     return(4);
   }
-  else 
-    printf("Loading %d Ansys elements from %s\n",noelements,filename);
+   
+  if(info) printf("Loading %d Ansys elements from %s\n",noelements,filename);
 
   for(j=1;j<=noelements;j++) {
 
-    getline; cp=line;
+    getline; 
+    cp=line;
 
     for(i=0;i<8;i++) {
       ind = next_int(&cp);
@@ -1617,15 +1782,14 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
     if(ansystypes[k] != currenttype) k=1;
 
     if(ansysnodes[k] > 8) {
-      getline; cp=line;
+      getline; 
+      cp=line;
 
       if(ansysnodes[k] == 10 && topology[2] != topology[3])
 	imax = 10;
       else 
 	imax = 20;
 
-      if(debug) printf("imax = %d\n",imax);
-
       for(i=8;i<imax;i++) {
 	ind = next_int(&cp);
 	if(cp[0] == '.') cp++;
@@ -1633,31 +1797,13 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
       }
     }
 
-#if 0
-    printf("topology %3d : ",j);
-    for(i=0;i<imax;i++) 
-      printf("%3d ",topology[i]);
-    printf("\n");
-#endif
-
     ReorderAnsysNodes(data,&topology[0],j,ansysdim[k],ansysnodes[k]);
-
-#if 0
-    if(j<10) {
-      printf("elem=%d dim=%d nodes=%d\n",j,ansysdim[k],ansysnodes[k]);
-      for(i=0;i<ansysnodes[k];i++) {
-	l = data->topology[j][i],
-	printf("i=%d ind=%d x=%.3lg y=%.3lg z=%.3lg\n",
-	       i,l,data->x[l],data->y[l],data->z[l]);
-      }	  
-    }
-#endif
   }      
-
-
-
   fclose(in);
 
+
+  /* ExportMesh.boundary */
+
   sprintf(filename,"%s.boundary",prefix);
   printf("Calculating nodes in file %s\n",filename);
   if ((in = fopen(filename,"r")) == NULL) {
@@ -1669,34 +1815,27 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
   j = 0;
   for(i=0;getline;i++)
     if(!strstr(line,"Boundary")) j++;
-  fclose(in);
 
   boundarynodes = j;
   nosides = 2*boundarynodes;
 
-  if(info) printf("There are %d boundary nodes, thus allocating %d elements\n",
+  if(info) printf("There are %d boundary nodes, allocating %d elements\n",
 		  boundarynodes,nosides);
   AllocateBoundary(bound,nosides);
   nodeindx = Ivector(1,boundarynodes);
   boundindx = Ivector(1,boundarynodes);
 
-
-  j++;
-
-  if ((in = fopen(filename,"r")) == NULL) {
-    printf("LoadAnsysInput: The opening of the boundary-file %s failed!\n",
-	   filename);
-  }
-
   if(info) printf("Loading Ansys boundary from %s\n",filename);
 
-  for(i=1;i<=boundarynodes;i++) 
-    nodeindx[i] = boundindx[i] = 0;
+  for(i=1;i<=boundarynodes;i++) nodeindx[i] = boundindx[i] = 0;
 
+  rewind(in);
+  maxside = 0;
   j = 0;
   for(i=0;getline;i++) {
     if(strstr(line,"Boundary")) {
       sscanf(line,"%d",&sidetype);
+      maxside = MAX(sidetype,maxside);
     }
     else {
       j++;
@@ -1706,24 +1845,109 @@ int LoadAnsysInput(struct FemType *data,struct BoundaryType *bound,
       boundindx[j] = sidetype;
     }
   }
-  printf("Found %d nodes on boundary %s\n",j,filename);
+  printf("Found %d boundary nodes with %d as maximum side.\n",j,maxside);
   fclose(in);
-    
+
   FindPointParents(data,bound,boundarynodes,nodeindx,boundindx,info);
 
+  if(namesexist) {
+    int bc,bcind,bcind0,*bctypes,*bctypeused,*bcused,newsides,unused1;
+
+    data->bodynamesexist = TRUE;
+    if(bound[0].nosides) {
+      newsides = 0;
+      bctypes = Ivector(1,maxside);
+      bctypeused = Ivector(1,maxside);
+      bcused = Ivector(1,bound[0].nosides);      
+      for(i=1;i<=bound[0].nosides;i++) bcused[i] = FALSE;      
+      data->boundarynamesexist = TRUE;
+      for(i=1;i<=maxside;i++) 
+	bctypeused[i] = FALSE;      
+    }
+
+    sprintf(filename,"%s.names",prefix);
+    in = fopen(filename,"r");
+            
+    for(;;) {
+      if(Getrow(line,in,TRUE)) break;
+      sscanf(line,"%d%s%s%d",&bcind,&text[0],&text2[0],&sides);
+      
+      if(strstr(text2,"BODY")) {      
+	getline;
+	sscanf(line,"%d%d",&j,&bcind);
+	strcpy(data->bodyname[bcind],text);	
+      }
+      else if(strstr(text2,"BOUNDARY")) {      
+	/* Read the boundary groups belonging to a particular name */
+	for(i=1;i<=maxside;i++) 
+	  bctypes[i] = 0;
+	for(i=1;i<=sides;i++) {
+	  getline;
+	  sscanf(line,"%d%d",&j,&bcind);
+	  bctypes[bcind] = TRUE;
+	}
+	
+	/* Find 1st unsed boundarytype */
+	for(i=1;i<=maxside;i++) 
+	  if(bctypes[i] && !bctypeused[i]) break;
+	
+	bcind = i;
+	bctypeused[bcind] = TRUE;
+	if(0) printf("First unused boundary is of type %d\n",bcind);
+	strcpy(data->boundaryname[bcind],text);
+	
+	/* Check which of the BCs have already been named */
+	k = l = 0;
+	for(i=1;i<=bound[0].nosides;i++) {
+	  j = bound[0].types[i];
+	  
+	  /* The bc is not given any name, hence it can't be a duplicate */
+	  if(!bctypes[j]) continue;
+	  
+	  if(!bcused[i]) {
+	    k++;
+	    bcused[i] = bcind;
+	  }
+	  else {
+	    l++;
+	    if(newsides == 0) AllocateBoundary(&bound[1],bound[0].nosides);
+	    newsides++;
+	    bound[1].types[newsides] = bcind;
+	    bound[1].parent[newsides] =  bound[0].parent[i];
+	    bound[1].side[newsides] = bound[0].side[i];
+	    bound[1].parent2[newsides] = bound[0].parent2[i];
+	    bound[1].side2[newsides] = bound[0].side2[i];
+	    bound[1].normal[newsides] = bound[0].normal[i];
+	  }
+	}
+	if(info) printf("There are %d boundary elements with name %s.\n",k+l,data->boundaryname[bcind]);
+      }
+    }
+
+    fclose(in);
+
+    /* Put the indexes of all conditions with the same name to be same */
+    if(bound[0].nosides) {
+      for(i=1;i<=bound[0].nosides;i++) 
+	if(bcused[i]) bound[0].types[i] = bcused[i];
+      if(newsides) {
+	bound[1].nosides = newsides;
+	if(info) printf("Created %d additional boundary elements to achieve unique naming.\n",newsides);
+      }
+      free_Ivector(bctypes,1,maxside);    
+      free_Ivector(bctypeused,1,maxside);    
+      free_Ivector(bcused,1,bound[0].nosides);
+    }
+  }
+
   free_Ivector(boundindx,1,boundarynodes);
   free_Ivector(nodeindx,1,boundarynodes);
 
-#if 0
-  chdir("..");
-#endif
-
   return(0);
 }
 
 
 
-
 static int FindFemlabParents(struct FemType *data,struct BoundaryType *bound,
 			    int nosides,int sidenodes,int *boundindx,int **nodeindx)
 {
@@ -1896,10 +2120,9 @@ int LoadFemlabMesh(struct FemType *data,struct BoundaryType *bound,
 
   fclose(in);
 
-  printf("Femlab (%d-dim) file has %d %d-node elements, %d nodes and %d sides.\n",
+  printf("Femlab %dD file has %d %d-node elements, %d nodes and %d sides.\n",
 	 dim,noelements,nonodes,noknots,nosides);
   
-
   sprintf(filename,"%s.node",prefix);
   if ((in = fopen(filename,"r")) == NULL) {
     printf("LoadFemlabInput: The opening of the nodes-file %s failed!\n",
@@ -1940,7 +2163,6 @@ int LoadFemlabMesh(struct FemType *data,struct BoundaryType *bound,
   fclose(in);
 
 
-
   sprintf(filename,"%s.elem",prefix);
   if ((in = fopen(filename,"r")) == NULL) {
     printf("LoadFemlabInput: The opening of the element-file %s failed!\n",
@@ -2820,6 +3042,30 @@ end:
 }
 
 
+
+static void ReorderComsolNodes(int elementtype,int *topo) 
+{
+  int i,j,tmptopo[MAXNODESD2];
+  int order404[]={1,2,4,3};
+ 
+   
+  switch (elementtype) {
+ 
+  case 404:
+    for(i=0;i<elementtype%100;i++) 
+      tmptopo[i] = topo[i];
+    for(i=0;i<elementtype%100;i++) 
+      topo[i] = tmptopo[order404[i]-1];
+    break;
+
+  default:
+    break;
+
+  }  
+}
+
+
+
 int LoadComsolMesh(struct FemType *data,char *prefix,int info)
 /* Load the grid in Comsol Multiphysics mesh format */
 {
@@ -2891,7 +3137,10 @@ omstart:
       if(strstr(line,"vtx")) elembasis = 100;
       if(strstr(line,"edg")) elembasis = 200;
       if(strstr(line,"tri")) elembasis = 300;
+      if(strstr(line,"quad")) elembasis = 400;
       if(strstr(line,"tet")) elembasis = 500;
+      if(strstr(line,"prism")) elembasis = 700;
+     
       if(debug) printf("elembasis=%d\n",elembasis);     
     }
 
@@ -2943,11 +3192,14 @@ omstart:
 
 	noelements = noelements + 1;
 	if(allocated) {
+	  data->elementtypes[noelements] = elemtype;
+	  data->material[noelements] = 1;	  
 	  cp = line;
 	  for(j=0;j<elemnodes;j++)
 	    data->topology[noelements][j] = next_int(&cp) + offset;
-	  data->elementtypes[noelements] = elemtype;
-	  data->material[noelements] = 1;	  
+
+	  if(1) ReorderComsolNodes(elemtype,data->topology[noelements]);
+
 	}
       }
     }
@@ -2981,9 +3233,9 @@ omstart:
 	  if(elemdim < dim) {
 	    if(minbc > material) minbc = material;
 	  }
-	  else {
+	  else { 
 	    if(mindom > material) mindom = material;	  
-	  }	  
+	  }
 	}
 
       }
@@ -3002,7 +3254,7 @@ end:
     if(noknots == 0 || noelements == 0 || maxnodes == 0) {
        printf("Invalid mesh consits of %d knots and %d %d-node elements.\n",
 	     noknots,noelements,maxnodes);     
-       close(in);
+       fclose(in);
        return(2);
     }
 
@@ -3021,7 +3273,7 @@ end:
 
     goto omstart;    
   }
-  close(in);
+  fclose(in);
 
   if(info) printf("The Comsol mesh was loaded from file %s.\n\n",filename);
   return(0);
@@ -3083,6 +3335,29 @@ static int GmshToElmerType(int gmshtype)
 }
 
 
+static void GmshToElmerIndx(int elemtype,int elemind[])
+{
+  int tmpind[MAXNODESD2];
+
+  switch (elemtype) {
+      
+  case 510:        
+    tmpind[8] = elemind[8];
+    tmpind[9] = elemind[9];
+    elemind[8] = tmpind[9];
+    elemind[9] = tmpind[8];    
+    break;
+
+    /* There seems to be conflicting data whether this is needed or not */
+  case 306:        
+    tmpind[4] = elemind[4];
+    tmpind[5] = elemind[5];
+    elemind[4] = tmpind[5];
+    elemind[5] = tmpind[4];    
+    break;
+  }
+
+}
 
 
 
@@ -3193,7 +3468,12 @@ allocate:
 		   elementtype,elemnodes);
 	  }	  
 	  for(j=0;j<elemnodes;j++)
-	    data->topology[i][j] = next_int(&cp);
+	    elemind[j] = next_int(&cp);
+	  
+	  GmshToElmerIndx(elementtype,elemind);	  
+
+	  for(j=0;j<elemnodes;j++)
+	    data->topology[i][j] = elemind[j];
 	}
 	else {
 	  maxnodes = MAX(elemnodes,maxnodes);
@@ -3202,7 +3482,7 @@ allocate:
       }
       getline;
       if(!strstr(line,"$ENDELM")) 
-	printf("NOD section should end to string ENDELM\n");
+	printf("ELM section should end to string ENDELM\n");
     }
 
   }
@@ -3216,6 +3496,7 @@ allocate:
     goto allocate;
   }
 
+
   if(maxindx > noknots) {
     printf("Renumbering the Gmsh nodes from %d to %d\n",maxindx,noknots);
 
@@ -3237,8 +3518,7 @@ allocate:
   }
 
   ElementsToBoundaryConditions(data,bound,info);
-  ReorderTypes(data,bound,info);
- 
+
   printf("Succesfully read the mesh from the Gmsh input file.\n");
 
   return(0);
