@@ -36,6 +36,8 @@ int SetDiscontinuousBoundary(struct FemType *data,struct BoundaryType *bound,
 			     int boundtype,int endnodes,int info);
 int SetDiscontinuousPoints(struct FemType *data,struct PointType *point,
 			   int material);
+int SetConnectedBoundary(struct FemType *data,struct BoundaryType *bound,
+			 int bctype,int connecttype,int info);
 int FindCorners(struct GridType *grid,struct CellType *cell,
 		struct FemType *data,int info);
 
@@ -61,7 +63,9 @@ int SideToBulkElements(struct FemType *data,struct BoundaryType *bound,
 		       int *sidebulk,int sidetobulk,int info);
 void ReorderElements(struct FemType *data,struct BoundaryType *bound,
 		    int manual,Real corder[],int info);
-void ReorderTypes(struct FemType *data,struct BoundaryType *bound,int info);
+void RenumberBoundaryTypes(struct FemType *data,struct BoundaryType *bound,
+			   int renumber, int bcoffset, int info);
+void RenumberMaterialTypes(struct FemType *data,struct BoundaryType *bound,int info);
 void CreateKnotsExtruded(struct FemType *dataxy,struct BoundaryType *boundxy,
 			 struct GridType *grid,
 			 struct FemType *data,struct BoundaryType *bound,
@@ -73,7 +77,6 @@ void MergeElements(struct FemType *data,struct BoundaryType *bound,
 		   int manual,Real corder[],Real eps,int mergebounds,int info);
 void MergeBoundaries(struct FemType *data,struct BoundaryType *bound,int *doubles,int info);
 void SeparateCartesianBoundaries(struct FemType *data,struct BoundaryType *bound,int info);
-
 void ElementsToBoundaryConditions(struct FemType *data,
 				  struct BoundaryType *bound,int info);
 int FindPeriodicNodes(struct FemType *data,int periodicdim[],int info);
@@ -87,3 +90,8 @@ int CreateBoundaryLayer(struct FemType *data,struct BoundaryType *bound,
 			int nolayers, int *layerbounds, int *layernumber,
 			Real *layerratios, Real *layerthickness, int *layerparents,
 			int maxfilters, Real layereps, int info);
+int CreateBoundaryLayerDivide(struct FemType *data,struct BoundaryType *bound,
+			      int nolayers, int *layerbounds, int *layernumber,
+			      Real *layerratios, Real *layerthickness, int *layerparents,int info);
+int RotateTranslateScale(struct FemType *data,struct ElmergridType *eg,int info);
+int RemoveLowerDimensionalBoundaries(struct FemType *data,struct BoundaryType *bound,int info);
