@@ -1,4 +1,30 @@
- /* -----------------------:  FEMMESH.C  :----------------------
+/*  
+   ElmerGrid - A simple mesh generation and manipulation utility  
+   Copyright (C) 1995- , CSC - Scientific Computing Ltd.   
+
+   Author: Peter Rback
+   Email: Peter.Raback@csc.fi
+   Address: CSC - Scientific Computing Ltd.
+            Keilaranta 14
+            02101 Espoo, Finland
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*/
+
+
+ /* -----------------------:  femmesh.c  :----------------------
 
    These subroutines are used to create a mesh. They operate on 
    variables that are of type MeshType and GridType. The routines 
@@ -1664,14 +1690,13 @@ void SetElementDivisionExtruded(struct GridType *grid,int info)
 	  nzmax = i;
 	}
       }
-      if(grid->autoratio && grid->xzratio * grid->dx0 > dzmax)
-	break;
+
+      if(grid->autoratio && grid->xzratio * grid->dx0 > dzmax) break;
+
       grid->zelems[nzmax] += 1;
       sumzelems++;
-      if(!grid->autoratio && sumzelems >= grid->totzelems)
-	break;
-      
-      if(0) printf("dzmax = %.3lg\n",dzmax);
+
+      if(!grid->autoratio && sumzelems >= grid->totzelems) break;
     }
   }
 
