@@ -32,10 +32,15 @@
 *  3, 6 and 10-node triangles. There is also limited 3D functionality       *
 *  with 8, 20 and 27-node cubes and 6-node prisms.                          *
 *                                                                           *
-*  Note: this software was initially part of Pirfem code, then later        *
-*  called Quickmesh, and finally renamed to Elmergrid. The code has never   *
-*  been designed and with new features the code has eventually become very  *
-*  dirty.                                                                   *
+*  The program may also be used as a mesh import and export utility. It     *
+*  is able to read several different formats and writes mainly Elmer input  *
+*  and output formats. The meshes may also be given some simple operations. *
+*                                                                           *
+*  Note: this software was initially part of my first fem implementation    *
+*  the Pirfem code, then later called Quickmesh, and finally renamed to     *
+*  Elmergrid. The code has never been designed and with new features the    *
+*  code has eventually become very dirty and does not present my view of    *
+*  good programming.                                                        *
 *                                                                           *
 ****************************************************************************/
 
@@ -60,6 +65,30 @@
 #include "easymesh.h"
 
 
+#define MAXMETHODS 17
+char *IOmethods[] = {
+  /*0*/ "EG",
+  /*1*/ "ELMERGRID",
+  /*2*/ "ELMERSOLVER",
+  /*3*/ "ELMERPOST",
+  /*4*/ "ANSYS",
+  /*5*/ "IDEAS",
+  /*6*/ "ABAQUS",
+  /*7*/ "FIDAP",
+  /*8*/ "EASYMESH",
+  /*9*/ "COMSOL",
+  /*10*/ "FIELDVIEW",
+  /*11*/ "TRIANGLE",
+  /*12*/ "MEDIT",
+  /*13*/ "GID",
+  /*14*/ "GMSH",
+  /*15*/ "PARTITIONED",
+  /*16*/ "NASTRAN",
+  /*17*/ "FASTCAP"
+};
+
+
+
 
 static void Instructions()
 {
@@ -83,31 +112,28 @@ static void Instructions()
   printf("4)  .ansys    : Ansys input format\n");
   printf("5)  .inp      : Abaqus input format by Ideas\n");
   printf("6)  .fil      : Abaqus output format\n");
-  printf("7)  .FDNEUT   : Fidap neutral file\n");
+  printf("7)  .FDNEUT   : Gambit (Fidap) neutral file\n");
   printf("8)  .d        : Easymesh input format\n");
-  printf("9)  .mphtxt   : Comsol Multiphysics (was Femlab) mesh format\n");
+  printf("9)  .mphtxt   : Comsol Multiphysics mesh format\n");
   printf("10) .dat      : Fieldview format\n");
   printf("11) .node,.ele: Triangle 2D mesh format\n");
   printf("12) .mesh     : Medit mesh format\n");
   printf("13) .msh      : GID mesh format\n");
   printf("14) .msh      : Gmsh mesh format\n");
   printf("15) .ep.i     : Partitioned ElmerPost format\n");
+#if 0
+  printf("16) .msh      : Nastran format\n");
+#endif 
 
   printf("\nThe second parameter defines the output file format:\n");
   printf("1)  .grd      : ElmerGrid file format\n");
   printf("2)  .mesh.*   : ElmerSolver format (also partitioned .part format)\n");
   printf("3)  .ep       : ElmerPost format\n");
-
 #if 0
-  printf("5)  .n .e .s  : Easymesh output format\n");
-  printf("6)  .inp      : Abaqus input format\n");
+  printf("5)  .inp      : Abaqus input format\n");
   printf("7)  .fidap    : Fidap format\n");
-  printf("8)  .side     : free boundaries\n");
-  printf("9) .vf        : view factors of a boundary\n");
-  printf("10) .map      : mapping matrix between Elmergrid grids\n");
-  printf("11) .ep       : Elmer to Elmer mapping.\n");
-  printf("12) .ep       : Elmer to Elmer mapping with mapping file.\n");
-  printf("13) .ep       : Fastcap input format.\n");
+  printf("8)  .n .e .s  : Easymesh output format\n");
+  printf("17) .ep       : Fastcap input format.\n");
 #endif
 
   printf("\nThe third parameter is the name of the input file.\n");
@@ -134,25 +160,35 @@ static void Instructions()
   printf("-cylinder            : map 2D/3D cylindrical mesh to a cartesian mesh\n");
   printf("-reduce int[2]       : reduce element order at material interval [int1 int2]\n");
   printf("-increase            : increase element order from one to two\n");
+#if 0
+  /* This functionality has moved into the ElmerSolver */
   printf("-pelem int[3]        : p-elements of power int3 at interval [int1 int2]\n");
   printf("-belem int[3]        : set bubble dofs to int3 at interval [int1 int2]\n");
+#endif
   printf("-partition int[3]    : the mesh will be partitioned in main directions\n");
   printf("-partorder real[3]   : in the above method, the direction of the ordering\n");
 #if PARTMETIS
-  printf("-metis int           : the mesh will be partitioned with Metis\n");
+  printf("-metis int[2]        : the mesh will be partitioned with Metis\n");
 #endif
-  printf("-periodic int[3]     : declere the periodic coordinate directions\n");
+  printf("-periodic int[3]     : decleare the periodic coordinate directions\n");
   printf("-bcoffset int        : add an offset to the boundary conditions\n");
   printf("-discont int         : make the boundary to have secondary nodes\n");
+  printf("-connect int         : make the boundary to have internal connection among its elements\n");
+  printf("-removelowdim        : remove boundaries that are two ranks lower than highest dim\n");
+  printf("-bulkorder           : renumber materials types from 1 so that every number is used\n");
+  printf("-boundorder          : renumber boundary types from 1 so that every number is used\n");
+  printf("-autoclean           : this performs the united action of the three above\n");
   printf("-bulkbound int[3]    : set the union of materials [int1 int2] to be boundary int3\n");
   printf("-boundbound int[3]   : set the union of boundaries [int1 int2] to be boundary int3\n");
   printf("-bulktype int[3]     : set material types in interval [int1 int2] to type int3\n");
   printf("-boundtype int[3]    : set sidetypes in interval [int1 int2] to type int3\n");
   printf("-layer int[2] real[2]: make a boundary layer for given boundary\n");
   printf("-layermove int       : apply Jacobi filter int times to move the layered mesh\n");
-  printf("-3d / -2d / -1d      : mesh is 3, 2 or 1-dimensional\n");
+  printf("-divlayer int[2] real[2]: make a boundary layer for given boundary\n");
+  printf("-3d / -2d / -1d      : mesh is 3, 2 or 1-dimensional (applies to examples)\n");
   printf("-isoparam            : ensure that higher order elements are convex\n");
-  printf("-nobound             : disable saving of boundary elements\n");
+  printf("-nobound             : disable saving of boundary elements in ElmerPost format\n");
+  printf("-names               : conserve name information where applicable\n");
 #if 0
   printf("-map str             : file with mapping info for mesh-to-mesh interpolation\n");
 #endif
@@ -179,6 +215,7 @@ void InitParameters(struct ElmergridType *eg)
   eg->rotate = FALSE;
   eg->polar = FALSE;
   eg->cylinder = FALSE;
+  eg->usenames = FALSE;
   eg->layers = 0;
   eg->layereps = 0.0;
   eg->layermove = 0;
@@ -190,6 +227,7 @@ void InitParameters(struct ElmergridType *eg)
   eg->increase = FALSE;
   eg->translate = FALSE;
   eg->isoparam = FALSE;
+  eg->removelowdim = FALSE;
   eg->dim = 3;
   eg->scale = FALSE;
   eg->order = FALSE;
@@ -206,6 +244,8 @@ void InitParameters(struct ElmergridType *eg)
   eg->periodicdim[0] = 0;
   eg->periodicdim[1] = 0;
   eg->periodicdim[2] = 0;
+  eg->bulkorder = FALSE;
+  eg->boundorder = FALSE;
   eg->sidemappings = 0;
   eg->bulkmappings = 0;
   eg->clone[0] = eg->clone[1] = eg->clone[2] = 0;
@@ -213,6 +253,7 @@ void InitParameters(struct ElmergridType *eg)
   eg->mirrorbc = 0;
   eg->decimals = 12;
   eg->discont = 0;
+  eg->connect = 0;
   eg->advancedmat = 0;
   
   for(i=0;i<MAXSIDEBULK;i++) 
@@ -223,42 +264,46 @@ void InitParameters(struct ElmergridType *eg)
 int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 {
   int arg,i,dim;
+  char command[MAXLINESIZE];
   
   dim = eg->dim;
 
   printf("Elmergrid reading in-line arguments\n");
 
-  if(strcmp(argv[1],"eg") == 0) eg->inmethod = 0;
-  else if(strcmp(argv[1],"ElmerGrid") == 0) eg->inmethod = 1;
-  else if(strcmp(argv[1],"ElmerSolver") == 0) eg->inmethod = 2;
-  else if(strcmp(argv[1],"ElmerPost") == 0) eg->inmethod = 3;
-  else if(strcmp(argv[1],"Ansys") == 0) eg->inmethod = 4;
-  else if(strcmp(argv[1],"Ideas") == 0) eg->inmethod = 5;
-  else if(strcmp(argv[1],"Abaqus") == 0) eg->inmethod = 6;
-  else if(strcmp(argv[1],"Fidap") == 0) eg->inmethod = 7;
-  else if(strcmp(argv[1],"Easymesh") == 0) eg->inmethod = 8;
-  else if(strcmp(argv[1],"Femlab") == 0) eg->inmethod = 9;
-  else if(strcmp(argv[1],"Comsol") == 0) eg->inmethod = 9;
-  else if(strcmp(argv[1],"Fieldview") == 0) eg->inmethod = 10;
-  else if(strcmp(argv[1],"Triangle") == 0) eg->inmethod = 11;
-  else if(strcmp(argv[1],"Medit") == 0) eg->inmethod = 12;
-  else if(strcmp(argv[1],"Gid") == 0) eg->inmethod = 13;
-  else if(strcmp(argv[1],"Gmsh") == 0) eg->inmethod = 14;
-  else if(strcmp(argv[1],"Partitioned") == 0) eg->inmethod = 15;
-  else eg->inmethod = atoi(argv[1]);
+  /* Type of input file */
+  strcpy(command,argv[1]);
+  for(i=0;i<MAXLINESIZE;i++) command[i] = toupper(command[i]);
+  for(i=0;i<=MAXMETHODS;i++) {
+    if(strstr(command,IOmethods[i])) {
+      eg->inmethod = i;
+      break;
+    }
+  }
+  if(i>MAXMETHODS) eg->inmethod = atoi(argv[1]);
 
-  if(strcmp(argv[2],"ElmerGrid") == 0) eg->outmethod = 1;
-  else if(strcmp(argv[2],"ElmerSolver") == 0) eg->outmethod = 2;
-  else if(strcmp(argv[2],"ElmerPost") == 0) eg->outmethod = 3;
-  else if(strcmp(argv[2],"Fastcap") == 0) eg->outmethod = 13;
-  else eg->outmethod = atoi(argv[2]);
 
+  /* Type of output file (fewer options) */
+  strcpy(command,argv[2]);
+  for(i=0;i<MAXLINESIZE;i++) command[i] = toupper(command[i]);
+  for(i=1;i<=MAXMETHODS;i++) {
+    if(strstr(command,IOmethods[i])) {
+      eg->outmethod = i;
+      break;
+    }
+  }
+  if(i>MAXMETHODS) eg->outmethod = atoi(argv[2]);
+ 
+
+  /* Name of output file */
   strcpy(eg->filesin[0],argv[3]);
   strcpy(eg->filesout[0],eg->filesin[0]);
   strcpy(eg->mapfile,eg->filesin[0]);
 
 
+  /* The optional inline parameters */
+
   for(arg=4;arg <argc; arg++) {
+
     if(strcmp(argv[arg],"-in") ==0 ) {
       if(arg+1 >= argc) {
 	printf("The secondary input file name is required as a paramater\n");
@@ -270,6 +315,7 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 	eg->nofilesin++;
       }
     }
+
     if(strcmp(argv[arg],"-out") == 0) {
       if(arg+1 >= argc) {
 	printf("The output name is required as a paramater\n");
@@ -312,6 +358,7 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 	if(dim==3) eg->corder[2] = atof(argv[arg+3]);
       }
     }
+
     if(strcmp(argv[arg],"-autoorder") == 0) {
       eg->order = 2;
     }
@@ -398,11 +445,29 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
       }
     }
 
-    if(strcmp(argv[arg],"-unite") ==0) {
+    if(strcmp(argv[arg],"-unite") == 0) {
       eg->unitemeshes = TRUE;
       printf("The meshes will be united.\n");
     }   
 
+    if(strcmp(argv[arg],"-names") == 0) {
+      eg->usenames = TRUE;
+      printf("Names will be conserved when possible\n");
+    }   
+
+    if(strcmp(argv[arg],"-removelowdim") == 0) {
+      eg->removelowdim = TRUE;
+      printf("Lower dimensional boundaries will be removed\n");
+    }   
+
+    if(strcmp(argv[arg],"-autoclean") == 0) {
+      eg->removelowdim = TRUE;
+      eg->bulkorder = TRUE;
+      eg->boundorder = TRUE;
+      printf("Lower dimensional boundaries will be removed\n");
+      printf("Materials and boundaries will be renumbered\n");
+    }   
+
     if(strcmp(argv[arg],"-polar") == 0) {
       eg->polar = TRUE;
       printf("Making transformation to polar coordinates.\n");
@@ -434,6 +499,12 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
     if(strcmp(argv[arg],"-increase") == 0) {
       eg->increase = TRUE;
     }
+    if(strcmp(argv[arg],"-bulkorder") == 0) {
+      eg->bulkorder = TRUE;
+    }
+    if(strcmp(argv[arg],"-boundorder") == 0) {
+      eg->boundorder = TRUE;
+    }
     if(strcmp(argv[arg],"-pelem") == 0) {
       for(i=arg+1;i<argc && strncmp(argv[i],"-",1); i++) 
 	eg->pelemmap[3*eg->pelems+i-1-arg] = atoi(argv[i]);
@@ -450,14 +521,16 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 	return(3);
       }
       else {
+	eg->partitions = 1;
 	eg->partdim[0] = atoi(argv[arg+1]);
 	eg->partdim[1] = atoi(argv[arg+2]);
 	if(dim == 3) eg->partdim[2] = atoi(argv[arg+3]);
 	eg->partitions = 1;
 	for(i=0;i<3;i++) {
-	  if(eg->partdim[i] < 1) eg->partdim[i] = 1;
+	  if(eg->partdim[i] == 0) eg->partdim[i] = 1;
 	  eg->partitions *= eg->partdim[i];
 	}
+	if(eg->partdim[0] < 0) eg->partdim[0] = -eg->partdim[0];
 	printf("The mesh will be partitioned with simple division to %d partitions.\n",
 	       eg->partitions);
       }
@@ -484,6 +557,12 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
       else {
 	eg->metis = atoi(argv[arg+1]);
 	printf("The mesh will be partitioned with Metis to %d partitions.\n",eg->metis);
+	if(arg+2 < argc) { 
+	  if(argv[arg+2][0] != '-') eg->metisopt = atoi(argv[arg+2]);
+	}
+	else {
+	  eg->metisopt = 0;
+	}
       }
 #else
       printf("This version of ElmerGrid was compiled without Metis library!\n");
@@ -512,6 +591,17 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 	eg->discont++;
       }
     }
+
+    if(strcmp(argv[arg],"-connect") == 0) {
+      if(arg+1 >= argc) {
+	printf("Give the connected boundary conditions.\n");
+ 	return(10);
+      }
+      else {
+	eg->connectbounds[eg->connect] = atoi(argv[arg+1]);
+	eg->connect++;
+      }
+    } 
  
     if(strcmp(argv[arg],"-boundbound") == 0) {
       for(i=arg+1;i<=arg+3 && i<argc; i++) {
@@ -554,6 +644,7 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 	eg->layers++;
       }
     }
+    
     if(strcmp(argv[arg],"-layermove") == 0) {
       if(arg+1 >= argc) {
 	printf("Give maximum number of Jacobi filters.\n");
@@ -564,6 +655,27 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
       }
     }
 
+    /* This uses a very dirty trick where the variables related to argument -layer are used 
+       with a negative indexing */ 
+    if(strcmp(argv[arg],"-divlayer") == 0) {
+      if(arg+4 >= argc) {
+	printf("Give four parameters for the layer: boundary, elements, relative thickness, ratio.\n");
+	return(19);
+      }
+      else if(abs(eg->layers) == MAXBOUNDARIES) {
+	printf("There can only be %d layers, sorry.\n",MAXBOUNDARIES);
+	return(20);
+      }
+      else {
+	eg->layerbounds[abs(eg->layers)] = atoi(argv[arg+1]);
+	eg->layernumber[abs(eg->layers)] = atoi(argv[arg+2]);
+	eg->layerthickness[abs(eg->layers)] = atof(argv[arg+3]);
+	eg->layerratios[abs(eg->layers)] = atof(argv[arg+4]);
+	eg->layerparents[abs(eg->layers)] = 0;
+	eg->layers--;
+      }
+    }
+
     if(strcmp(argv[arg],"-3d") == 0) {
       eg->dim = dim = 3;
     }
@@ -639,25 +751,39 @@ int InlineParameters(struct ElmergridType *eg,int argc,char *argv[])
 int LoadCommands(char *prefix,struct ElmergridType *eg,
 		 struct GridType *grid, int mode,int info) 
 {
-  char filename[MAXFILESIZE];
-  char command[MAXLINESIZE],params[MAXLINESIZE],*cp;
+  char filename[MAXFILESIZE],command[MAXLINESIZE],params[MAXLINESIZE],*cp;
 
   FILE *in;
   int i,j,k,l,error=0;
 
-  if(mode == 0) { 
-    AddExtension(prefix,filename,"eg");
-    if ((in = fopen(filename,"r")) == NULL) {
-      printf("LoadCommands: opening of the file '%s' wasn't succesfull !\n",filename);
+  if( mode == 0) {  
+    if ((in = fopen("ELMERGRID_STARTINFO","r")) == NULL) {
       return(1);
+    }
+    else {
+      fscanf(in,"%s",filename);
+      fclose(in);
+      printf("Using the file %s defined in ELMERGRID_STARTINFO\n",filename);
+      if ((in = fopen(filename,"r")) == NULL) {
+	printf("LoadCommands: opening of the file '%s' wasn't succesfull !\n",filename);
+	return(2);
+      }    
     }    
     if(info) printf("Loading ElmerGrid commands from file '%s'.\n",filename);    
   }
-  else if(mode == 1) {
+  if(mode == 1) { 
+    AddExtension(prefix,filename,"eg");
+    if ((in = fopen(filename,"r")) == NULL) {
+      printf("LoadCommands: opening of the file '%s' wasn't succesfull !\n",filename);
+      return(2);
+    }    
+    if(info) printf("Loading ElmerGrid commands from file '%s'.\n",filename);    
+  }
+  else if(mode == 2) {
     AddExtension(prefix,filename,"grd");
     if ((in = fopen(filename,"r")) == NULL) {
       printf("LoadCommands: opening of the file '%s' wasn't succesfull !\n",filename);
-      return(1);
+      return(3);
     }    
     if(info) printf("Loading ElmerGrid commands from file '%s'.\n",filename);
   }
@@ -671,44 +797,44 @@ int LoadCommands(char *prefix,struct ElmergridType *eg,
       goto end;
     }    
 
+    /* If the mode is the command file mode read also the file information from the command file. */
+
     if(mode == 0) {
       if(strstr(command,"INPUT FILE")) {
 	sscanf(params,"%s",&eg->filesin[0]);
       }
+
       else if(strstr(command,"OUTPUT FILE")) {
 	sscanf(params,"%s",&eg->filesout[0]);
       }
+
       else if(strstr(command,"INPUT MODE")) {
 	for(j=0;j<MAXLINESIZE;j++) params[j] = toupper(params[j]);
-
-	if(strstr(params,"ELMERGRID")) eg->inmethod = 1;
-	else if(strstr(params,"ELMERSOLVER")) eg->inmethod = 2;
-	else if(strstr(params,"ELMERPOST")) eg->inmethod = 3;
-	else if(strstr(params,"ANSYS")) eg->inmethod = 4;
-	else if(strstr(params,"IDEAS")) eg->inmethod = 5;
-	else if(strstr(params,"ABAQUS")) eg->inmethod = 6;
-	else if(strstr(params,"FIDAP")) eg->inmethod = 7;
-	else if(strstr(params,"EASYMESH")) eg->inmethod = 8;
-	else if(strstr(params,"FEMLAB")) eg->inmethod = 9;
-	else if(strstr(params,"COMSOL")) eg->inmethod = 9;
-	else if(strstr(params,"FIELDVIEW")) eg->inmethod = 10;
-	else if(strstr(params,"TRIANGLE")) eg->inmethod = 11;
-	else if(strstr(params,"MEDIT")) eg->inmethod = 12;
-	else if(strstr(params,"GID")) eg->inmethod = 13;
-	else if(strstr(params,"GMSH")) eg->inmethod = 14;
-	else if(strstr(params,"PARTITIONED")) eg->inmethod = 15;
-	else sscanf(params,"%d",&eg->inmethod);
+	
+	for(i=0;i<=MAXMETHODS;i++) {
+	  if(strstr(params,IOmethods[i])) {
+	    eg->inmethod = i;
+	    break;
+	  }
+	}
+	if(i>MAXMETHODS) sscanf(params,"%d",&eg->inmethod);
       }
+
       else if(strstr(command,"OUTPUT MODE")) {
 	for(j=0;j<MAXLINESIZE;j++) params[j] = toupper(params[j]);
-
-	if(strstr(params,"ELMERGRID")) eg->outmethod = 1;
-	else if(strstr(params,"ELMERSOLVER")) eg->outmethod = 2;
-	else if(strstr(params,"ELMERPOST")) eg->outmethod = 3;
-	else if(strstr(params,"FASTCAP")) eg->outmethod = 13;
-	else sscanf(params,"%d",&eg->outmethod);	
+	
+	/* Type of output file (fewer options) */
+	for(i=1;i<=MAXMETHODS;i++) {
+	  if(strstr(params,IOmethods[i])) {
+	    eg->outmethod = i;
+	    break;
+	  }
+	}
+	if(i>MAXMETHODS) sscanf(params,"%d",&eg->outmethod);	
       }
     }    
+    /* End of command file specific part */
+
 
     if(strstr(command,"DECIMALS")) {
       sscanf(params,"%d",&eg->decimals);
@@ -840,6 +966,13 @@ int LoadCommands(char *prefix,struct ElmergridType *eg,
       eg->belems = i;
       printf("Found %d definitions for bubble elements.\n",i);
     }
+    else if(strstr(command,"METIS OPTION")) {
+#if PARTMETIS
+      sscanf(params,"%d",&eg->metisopt);
+#else
+      printf("This version of ElmerGrid was compiled without Metis library!\n");
+#endif
+    }
     else if(strstr(command,"METIS")) {
 #if PARTMETIS
       sscanf(params,"%d",&eg->metis);
@@ -936,6 +1069,18 @@ int LoadCommands(char *prefix,struct ElmergridType *eg,
 	eg->layers = i;
       }
     }
+    else if(strstr(command,"REMOVE LOWER DIMENSIONAL BOUNDARIES")) {
+      for(j=0;j<MAXLINESIZE;j++) params[j] = toupper(params[j]);
+      if(strstr(params,"TRUE")) eg->removelowdim = TRUE; 
+    }
+    else if(strstr(command,"REORDER MATERIAL")) {
+      for(j=0;j<MAXLINESIZE;j++) params[j] = toupper(params[j]);
+      if(strstr(params,"TRUE")) eg->bulkorder = TRUE; 
+    }
+    else if(strstr(command,"REORDER BOUNDARY")) {
+      for(j=0;j<MAXLINESIZE;j++) params[j] = toupper(params[j]);
+      if(strstr(params,"TRUE")) eg->boundorder = TRUE; 
+    }
     else if(strstr(command,"DIMENSION")) {
       sscanf(params,"%d",&eg->dim);
     }
@@ -988,78 +1133,12 @@ end:
 }
 
 
-int RotateTranslateScale(struct FemType *data,struct ElmergridType *eg)
-{
-  int i,j,k;
-  Real x,y,z,xz,yz,yx,zx,zy,xy,cx,cy,cz;
-
-  if(eg->scale) {
-    printf("Scaling mesh with vector [%.3lg %.3lg %.3lg]\n",
-	   eg->cscale[0],eg->cscale[1],eg->cscale[2]);
-    for(i=1;i<=data->noknots;i++) {
-      data->x[i] *= eg->cscale[0]; 
-      data->y[i] *= eg->cscale[1]; 
-      if(data->dim == 3) data->z[i] *= eg->cscale[2]; 
-    }
-    printf("Scaling of mesh finished.\n");
-  }
-  
-  if(eg->rotate) {
-    printf("Rotating mesh with degrees [%.3lg %.3lg %.3lg]\n",
-	   eg->crotate[0],eg->crotate[1],eg->crotate[2]);
-    cx = FM_PI * eg->crotate[0]/180.0;
-    cy = FM_PI * eg->crotate[1]/180.0;
-    cz = FM_PI * eg->crotate[2]/180.0;
-
-    for(i=1;i<=data->noknots;i++) {
-
-      x = data->x[i];
-      if(data->dim >= 2) y = data->y[i];
-      else y = 0.0;
-      if(data->dim >= 3) z = data->z[i];
-      else z = 0.0;
-
-      xz = x*cos(cz) + y*sin(cz);
-      yz = -x*sin(cz) + y*cos(cz);
-      
-      if(data->dim == 3) {
-	yx = yz*cos(cx) + z*sin(cx);
-	zx = -yz*sin(cx) + z*cos(cx);
-	
-	zy = zx*cos(cy) + xz*sin(cy);
-	xy = -zx*sin(cy) + xz*cos(cy);
-	
-	data->x[i] = xy;
-	data->y[i] = yx;
-	data->z[i] = zy;
-      }	
-      else {
-	data->x[i] = xz;
-	data->y[i] = yz;  
-      }
-    }
-    printf("Rotation of mesh finished.\n");
-  }
-
-  if(eg->translate) {
-    printf("Translating the mesh with vector [%.3lg %.3lg %.3lg]\n",
-	   eg->ctranslate[0],eg->ctranslate[1],eg->ctranslate[2]);
-    for(i=1;i<=data->noknots;i++) {
-      data->x[i] += eg->ctranslate[0];
-      data->y[i] += eg->ctranslate[1];
-      if(data->dim == 3) data->z[i] += eg->ctranslate[2];
-    }
-    printf("Translation of mesh finished.\n");
-  }
-
-  return(0);
-}
 
 
 
 int main(int argc, char *argv[])
 {
-  int i,j,k,l,inmethod,outmethod,info,errorstat;
+  int i,j,k,l,inmethod,outmethod,info,errorstat,sides;
   int nogrids,nomeshes,nofile,dim,elementsredone=0;
   int nodes3d,elements3d;
   Real mergeeps;
@@ -1071,6 +1150,7 @@ int main(int argc, char *argv[])
   struct ElmergridType eg;
   long ii;
 
+
   printf("\nStarting program Elmergrid\n");
 
   InitParameters(&eg);
@@ -1078,8 +1158,8 @@ int main(int argc, char *argv[])
   InitGrid(grids);
   info = TRUE;
 
-  if(argc == 2) {
-    errorstat = LoadCommands(argv[1],&eg,grids,0,info);     
+  if(argc <= 2) {
+    errorstat = LoadCommands(argv[1],&eg,grids,argc-1,info);     
     if(errorstat) Goodbye();
   }
   else if(argc < 4) {
@@ -1133,7 +1213,7 @@ int main(int argc, char *argv[])
       printf("Because file %s didn't exist, it was created for you.\n",eg.filesin[nofile]);
       Goodbye();
     }
-    LoadCommands(eg.filesin[nofile],&eg,grids,1,info); 
+    LoadCommands(eg.filesin[nofile],&eg,grids,2,info); 
     break;
 
   case 2: 
@@ -1145,16 +1225,13 @@ int main(int argc, char *argv[])
     }
     if(LoadElmerInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],info))
       Goodbye();
-    nomeshes = nofile+1;
-    if(0 && boundaries[nofile]->created) {
-      data[nofile].noboundaries = 1;
-    }
+    nomeshes++;
     break;
 
   case 3: 
     if(LoadSolutionElmer(&(data[nofile]),TRUE,eg.filesin[nofile],info)) 
       Goodbye();
-    nomeshes = nofile+1;
+    nomeshes++;
     break;
 
   case 4:
@@ -1162,7 +1239,7 @@ int main(int argc, char *argv[])
       malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
     if(LoadAnsysInput(&(data[0]),boundaries[0],eg.filesin[nofile],info)) 
       Goodbye();
-    nomeshes = nofile+1;
+    nomeshes++;
     break;
 
   case 5: 
@@ -1174,13 +1251,13 @@ int main(int argc, char *argv[])
     }
     if(LoadAbaqusInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE)) 
       Goodbye();
-    nomeshes = nofile+1;
+    nomeshes++;
     break;
 
   case 6:
     if(LoadAbaqusOutput(&(data[nofile]),eg.filesin[nofile],TRUE))
       Goodbye();
-    nomeshes = nofile+1;
+    nomeshes++;
     break;
 
   case 7:
@@ -1192,37 +1269,34 @@ int main(int argc, char *argv[])
       boundaries[nofile][i].created = FALSE; 
       boundaries[nofile][i].nosides = 0;
     }
+    if(!eg.usenames) data[nofile].boundarynamesexist = data[nofile].bodynamesexist = FALSE;
     ElementsToBoundaryConditions(&(data[nofile]),boundaries[nofile],TRUE);
-
-    nomeshes = nofile+1;
+    RenumberBoundaryTypes(&data[nofile],boundaries[nofile],TRUE,0,info);
+  
+    nomeshes++;
     break;
 
   case 8: 
-    {
-      int sides;
-      InitializeKnots(&(data[nofile]));
-      if( Easymesh(argc,argv,&data[nofile].noknots,
-		   &data[nofile].noelements,&sides)) 
-	Goodbye();	
-
-      data[nofile].dim = 2;
-      data[nofile].coordsystem = COORD_CART2;
-      data[nofile].maxnodes = 3;
-
-      AllocateKnots(&(data[nofile]));
-      boundaries[nofile] = (struct BoundaryType*)
-	malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
-      for(i=0;i<MAXBOUNDARIES;i++) {
-	boundaries[nofile][i].created = FALSE; 
-	boundaries[nofile][i].nosides = 0;
-      }
-
-      if(EasymeshCopy(&(data[nofile]),boundaries[nofile]))
-	Goodbye();
-
-      nomeshes = nofile + 1;
+    InitializeKnots(&(data[nofile]));
+    if( Easymesh(argc,argv,&data[nofile].noknots,
+		 &data[nofile].noelements,&sides)) 
+      Goodbye();	
+    
+    data[nofile].dim = 2;
+    data[nofile].coordsystem = COORD_CART2;
+    data[nofile].maxnodes = 3;
+    
+    AllocateKnots(&(data[nofile]));
+    boundaries[nofile] = (struct BoundaryType*)
+      malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
+    for(i=0;i<MAXBOUNDARIES;i++) {
+      boundaries[nofile][i].created = FALSE; 
+      boundaries[nofile][i].nosides = 0;
     }
-  break;
+    if(EasymeshCopy(&(data[nofile]),boundaries[nofile]))
+      Goodbye();    
+    nomeshes++;
+    break;
 
  case 9:
     boundaries[nofile] = (struct BoundaryType*)
@@ -1233,7 +1307,6 @@ int main(int argc, char *argv[])
     }
    
     if(LoadComsolMesh(&(data[nofile]),eg.filesin[nofile],info)) {
-
       printf("\n***********************************************************************************\n");
       printf("The reading of Comsol mesh file seems to have failed\n");
       printf("Trying out a previous version that requires the use of savemesh.m utility in Matlab\n");
@@ -1242,12 +1315,9 @@ int main(int argc, char *argv[])
 
       if(LoadFemlabMesh(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],info)) 
 	Goodbye();
-      nomeshes = nofile + 1;      
     }
-
     ElementsToBoundaryConditions(&(data[nofile]),boundaries[nofile],TRUE);
-    nomeshes = nofile + 1;
-
+    nomeshes++;
     break;
 
   case 10:
@@ -1259,9 +1329,8 @@ int main(int argc, char *argv[])
       boundaries[nofile][i].created = FALSE; 
       boundaries[nofile][i].nosides = 0;
     }
-
     ElementsToBoundaryConditions(&(data[nofile]),boundaries[nofile],TRUE);
-    nomeshes = 1;
+    nomeshes++;
     break;
 
   case 11:
@@ -1273,7 +1342,7 @@ int main(int argc, char *argv[])
     }
     if (LoadTriangleInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE))
       Goodbye();
-    nomeshes = 1;
+    nomeshes++;
     break;
 
   case 12:
@@ -1285,7 +1354,7 @@ int main(int argc, char *argv[])
     }
     if (LoadMeditInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE))
       Goodbye();
-    nomeshes = 1;
+    nomeshes++;
     break;
 
   case 13:
@@ -1297,7 +1366,7 @@ int main(int argc, char *argv[])
     }
     if (LoadGidInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE))
       Goodbye();
-    nomeshes = 1;
+    nomeshes++;
     break;
 
   case 14:
@@ -1309,15 +1378,29 @@ int main(int argc, char *argv[])
     }
     if (LoadGmshInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE))
       Goodbye();
-    nomeshes = 1;
+    nomeshes++;
     break;
 
   case 15: 
+    if(info) printf("Partitioned solution is fused on-the-fly therefore no operation may be performed.\n");
     FuseSolutionElmerPartitioned(eg.filesin[nofile],eg.filesout[nofile],eg.decimals,info);
-    printf("Partitioned solution is not read in memory, thus exiting\n");
+    if(info) printf("Finishing with the fusion of partitioned Elmer solutions\n");
     Goodbye();
     break;
 
+  case 16:
+    boundaries[nofile] = (struct BoundaryType*)
+      malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
+    for(i=0;i<MAXBOUNDARIES;i++) {
+      boundaries[nofile][i].created = FALSE; 
+      boundaries[nofile][i].nosides = 0;
+    }
+    if (LoadNastranInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE))
+      Goodbye();
+    nomeshes++;
+    break;
+
+
   default:
     Instructions();
     Goodbye();
@@ -1343,14 +1426,15 @@ int main(int argc, char *argv[])
     for(k=0;k<nogrids;k++) {
 
       CreateCells(&(grids[k]),&(cell[k]),info);  
-
       CreateKnots(&(grids[k]),cell[k],&(data[k]),0,0);
 
       boundaries[k] = (struct BoundaryType*)
 	malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 
 
-      for(j=0;j<MAXBOUNDARIES;j++) 
+      for(j=0;j<MAXBOUNDARIES;j++) {
 	boundaries[k][j].created = FALSE;
+	boundaries[k][j].nosides = FALSE;
+      }
 
       if(grids[k].noboundaries > 0) {
 	for(j=0;j<grids[k].noboundaries;j++) {
@@ -1364,13 +1448,12 @@ int main(int argc, char *argv[])
 			 grids[k].boundext[j],grids[k].boundint[j],
 			 grids[k].boundsolid[j],grids[k].boundtype[j]); 	    
 	  }
-
 	}
       }
     }
-
   }
 
+  /* Make the discontinous boundary needed, for example, in poor thermal conduction */
   for(k=0;k<nomeshes;k++) {
     if(!eg.discont) {
       for(j=0;j<grids[k].noboundaries;j++) 
@@ -1379,26 +1462,33 @@ int main(int argc, char *argv[])
 	  eg.discont++;	  
 	}
     }
-
     if(eg.discont) {
       for(i=1;i<=eg.discont;i++) 
 	SetDiscontinuousBoundary(&(data[k]),boundaries[k],eg.discontbounds[i-1],2,info);
     }
   }
 
-
-  for(k=0;k<nomeshes;k++) {
+  /* Make a connected boundary (specific to Elmer format) needed in linear constraints */
+  for(k=0;k<nomeshes;k++) 
+    for(i=1;i<=eg.connect;i++) 
+      SetConnectedBoundary(&(data[k]),boundaries[k],eg.connectbounds[i-1],i,info);
+  
+  /* Divide quadrilateral meshes into triangular meshes */
+  for(k=0;k<nomeshes;k++) 
     if(nogrids && (eg.triangles || grids[k].triangles == TRUE))
       ElementsToTriangles(&data[k],boundaries[k],info);
-  }
 
-  if(eg.layers) {
+  /* Make a boundary layer with two different methods */
+  if(eg.layers > 0) 
     for(k=0;k<nomeshes;k++) 
       CreateBoundaryLayer(&data[k],boundaries[k],eg.layers,
 			  eg.layerbounds, eg.layernumber, eg.layerratios, eg.layerthickness,
 			  eg.layerparents, eg.layermove, eg.layereps, info);
-  }
-
+  else if(eg.layers < 0) 
+    for(k=0;k<nomeshes;k++) 
+      CreateBoundaryLayerDivide(&data[k],boundaries[k],abs(eg.layers),
+				eg.layerbounds, eg.layernumber, eg.layerratios, eg.layerthickness,
+				eg.layerparents, info);
 
   if(outmethod != 1 && dim != 2 && eg.dim != 2) { 
     j = MAX(nogrids,1);
@@ -1454,6 +1544,7 @@ int main(int argc, char *argv[])
     }
   }
 
+  /* If the original mesh was given in polar coordinates make the transformation into cartesian ones */
   for(k=0;k<nomeshes;k++) {
     if(eg.polar || data[k].coordsystem == COORD_POLAR) {
       if(!eg.polar) eg.polarradius = grids[k].polarradius;
@@ -1461,19 +1552,20 @@ int main(int argc, char *argv[])
     }
   }
 
+  /* If the original mesh was given in cylindrical coordinates make the transformation into cartesian ones */
   for(k=0;k<nomeshes;k++) {
     if(eg.cylinder || data[k].coordsystem == COORD_CYL) {
       CylinderCoordinates(&data[k],info);
     }
   }
 
+  /* Unite meshes if there are several of them */
   if(eg.unitemeshes) {
     for(k=1;k<nomeshes;k++)
       UniteMeshes(&data[0],&data[k],boundaries[0],boundaries[k],info);
     nomeshes = nogrids = 1;
   }
   
-
   if(eg.clone[0] || eg.clone[1] || eg.clone[2]) {
     for(k=0;k<nomeshes;k++) {
       CloneMeshes(&data[k],boundaries[k],eg.clone,eg.clonesize,FALSE,info);
@@ -1490,6 +1582,7 @@ int main(int argc, char *argv[])
     }
   }
 
+  /* Naming convection for the case of several meshes */
   if(nomeshes > 1) {
     strcpy(prefix,eg.filesout[0]);
     for(k=0;k<nomeshes;k++)
@@ -1506,22 +1599,19 @@ int main(int argc, char *argv[])
       ReduceElementOrder(&data[k],eg.reducemat1,eg.reducemat2);
   }
 
-  for(k=0;k<nomeshes;k++) {
+  for(k=0;k<nomeshes;k++) 
     if(eg.increase) IncreaseElementOrder(&data[k],TRUE);
-  }
  
   for(k=0;k<nomeshes;k++) {
-    if(eg.merge) {
+    if(eg.merge) 
       MergeElements(&data[k],boundaries[k],eg.order,eg.corder,eg.cmerge,FALSE,TRUE);
-    }
-    else if(eg.order) {
+    else if(eg.order) 
       ReorderElements(&data[k],boundaries[k],eg.order,eg.corder,TRUE);
-    }
-    if(eg.isoparam) {
+    if(eg.isoparam) 
       IsoparametricElements(&data[k],boundaries[k],TRUE,info);
-    }
   }  
 
+  /* This is mainly here for historical reasons */
   for(k=0;k<nomeshes;k++) 
     if(eg.findsides) 
       SideToBulkElements(&data[k],boundaries[k],eg.sidebulk,FALSE,info);
@@ -1554,6 +1644,7 @@ int main(int argc, char *argv[])
   }
 
 
+  /* Create a variable so that when saving data in ElmerPost format there is something to visualize */
   for(k=0;k<nomeshes;k++) {
     if(data[k].variables == 0) {
       CreateVariable(&data[k],1,1,0.0,"Number",FALSE);
@@ -1562,22 +1653,20 @@ int main(int argc, char *argv[])
     }
   }
 
-  for(k=0;k<nomeshes;k++) {
-    RotateTranslateScale(&data[k],&eg);
-  }
+  for(k=0;k<nomeshes;k++) 
+    RotateTranslateScale(&data[k],&eg,info);
 
-  if(eg.bcoffset) {
-    printf("Addiing offset of %d to the BCs\n",eg.bcoffset);
+  if(eg.removelowdim) 
+    for(k=0;k<nomeshes;k++)
+      RemoveLowerDimensionalBoundaries(&data[k],boundaries[k],info);
 
-    for(k=0;k<nomeshes;k++) {
-      for(j=0;j < MAXBOUNDARIES;j++) {
-	if(!boundaries[k][j].created) continue;
-	for(i=1; i <= boundaries[k][j].nosides; i++) 
-	  boundaries[k][j].types[i] += eg.bcoffset;
-      }
-    }
-    printf("Renumbering boundary types finished\n");
-  }
+  if(eg.boundorder || eg.bcoffset) 
+    for(k=0;k<nomeshes;k++) 
+      RenumberBoundaryTypes(&data[k],boundaries[k],eg.boundorder,eg.bcoffset,info);
+
+  if(eg.bulkorder) 
+    for(k=0;k<nomeshes;k++) 
+      RenumberMaterialTypes(&data[k],boundaries[k],info);
 
   if(eg.sidemappings) {
     int currenttype;
@@ -1625,31 +1714,41 @@ int main(int argc, char *argv[])
   }
 
 
-  for(k=0;k<nomeshes;k++) {
-    if(eg.periodicdim[0] || eg.periodicdim[1] || eg.periodicdim[2]) {
-      if(info) printf("There seems to be peridic boundaries\n");
+  for(k=0;k<nomeshes;k++) 
+    if(eg.periodicdim[0] || eg.periodicdim[1] || eg.periodicdim[2]) 
       FindPeriodicNodes(&data[k],eg.periodicdim,info);
-    }
-  } 
+
    
   for(k=0;k<nomeshes;k++) {
-    if(eg.partitions > 1)
-#if 1
-      PartitionSimple(&data[k],eg.partdim,eg.periodicdim,eg.partorder,eg.partcorder,info);	
-#else
+    if(eg.partitions > 1) 
+      PartitionSimpleElements(&data[k],eg.partdim,eg.periodicdim,eg.partorder,eg.partcorder,info);	
+    else if(eg.partitions < -1) { 
+      eg.partitions = abs( eg.partitions );
       PartitionSimpleNodes(&data[k],eg.partdim,eg.periodicdim,eg.partorder,eg.partcorder,info);	
-#endif
+    }
 #if PARTMETIS
-    else if(eg.metis > 1) 
-      PartitionMetis(&data[k],eg.metis,info);
+    if(eg.metis) {
+      if(eg.metisopt % 4 == 0) 
+	PartitionMetisElements(&data[k],eg.metis,info);
+      else
+	PartitionMetisNodes(&data[k],eg.metis,eg.metisopt % 4,info);      
+    }
 #endif
-    if(eg.partitions > 1 || eg.metis > 1) 
-      OptimizePartitioning(&data[k],boundaries[k],info);
+    if(eg.partitions || eg.metis ) {
+      int noopt = 0;
+      if(eg.metis) noopt = eg.metisopt / 4;
+      OptimizePartitioning(&data[k],boundaries[k],noopt,info);
+    }
   }
 
   if(eg.pelems || eg.belems || eg.advancedmat) {
     int currenttype;
 
+    printf("\n***********************************************************************************\n");
+    printf("The advanced elements block will become obsolite and the stuff for defining the\n");
+    printf("non-Lagrangian elements is moved into the ElmerSolver command file!\n");
+    printf("\n***********************************************************************************\n");
+
     for(k=0;k<nomeshes;k++) {
       data[k].pelemtypes = Ivector(1,data[k].noelements); 
       data[k].pelems = TRUE;
@@ -1734,28 +1833,44 @@ int main(int argc, char *argv[])
     break;
 
   case 3:
+    for(k=0;k<nomeshes;k++) 
+      SaveSolutionElmer(&data[k],boundaries[k],eg.saveboundaries ? MAXBOUNDARIES:0,
+			eg.filesout[k],eg.decimals,info);
+    break;
+
   case 4:
+    printf("The output number 4 still refers to ep-file but will become obsolite in time\n");
+    printf("Rather use number 3 for ElmerPost output format\n");
     for(k=0;k<nomeshes;k++) {
       SaveSolutionElmer(&data[k],boundaries[k],eg.saveboundaries ? MAXBOUNDARIES:0,
 			eg.filesout[k],eg.decimals,info);
     }
     break;
 
-  case 5:
-    EasymeshSave();
-    break;
-
   case 6:
-    for(k=0;k<nogrids;k++)
+    for(k=0;k<nomeshes;k++)
       SaveAbaqusInput(&data[k],eg.filesout[k],info); 
     break;
     
   case 7:
-    for(k=0;k<nogrids;k++)
+    for(k=0;k<nomeshes;k++)
       SaveFidapOutput(&data[k],eg.filesout[k],info,1,data[k].dofs[1]);
     break;
-    
+
   case 8:
+    EasymeshSave();
+    break;
+
+  case 17:    
+    for(k=0;k<nomeshes;k++) 
+      SaveFastcapInput(&data[k],boundaries[k],eg.filesout[k],eg.decimals,info);
+    break;
+
+
+    
+    /* Some obsolite special formats related to mapping, view factors etc. */
+    
+  case 101:
     for(k=0;k<nogrids;k++) {   
       for(i=0;i<grids[k].noboundaries;i++)
 	if(boundaries[k][i].created == TRUE) {
@@ -1765,7 +1880,7 @@ int main(int argc, char *argv[])
     }
     break;
     
-  case 9:
+  case 102:
     for(k=0;k<nogrids;k++) {   
       for(i=0;i<grids[k].noboundaries;i++)
 	if(boundaries[k][i].created == TRUE) {
@@ -1779,8 +1894,8 @@ int main(int argc, char *argv[])
 	}
     }
     break;
-    
-  case 10:
+
+  case 103:
     if(nogrids <= 1) printf("No mapping possible for %d grid.\n",nogrids);
     for(k=0;k<nogrids-1;k++) {
       sprintf(prefix,"%s%dto%d",eg.filesout[0],k+1,k+2);
@@ -1790,7 +1905,7 @@ int main(int argc, char *argv[])
     }
     break;
 
-  case 11:
+  case 104:
     if(LoadSolutionElmer(&(data[1]),FALSE,eg.filesin[1],info)) {
       printf("The reading of the input file %s was not succesfull\n",eg.filesin[1]);
       Goodbye();
@@ -1800,7 +1915,7 @@ int main(int argc, char *argv[])
     SaveSolutionElmer(&(data[1]),boundaries[0],0,prefix,eg.decimals,TRUE);
     break;
 
-  case 12:
+  case 105:
     if(LoadSolutionElmer(&(data[1]),FALSE,eg.filesout[1],info)) {
       printf("The reading of the input file %s was not succesfull\n",eg.filesout[1]);
       Goodbye();
@@ -1811,11 +1926,6 @@ int main(int argc, char *argv[])
     SaveSolutionElmer(&(data[1]),boundaries[0],0,prefix,eg.decimals,TRUE);
     break;
 
-  case 13:    
-    for(k=0;k<nomeshes;k++) {
-      SaveFastcapInput(&data[k],boundaries[k],eg.filesout[k],eg.decimals,info);
-    }
-    break;
 
 
   default:
