@@ -15,7 +15,16 @@
 #define MAXVARS 20          /* maximum number of variables at the sides */
 #define MAXNODESD2 27       /* maximum number of 2D nodes */ 
 #define MAXNODESD1 9        /* maximum number of 1D nodes */
-#define MAXMAPPINGS 10   
+#define MAXMAPPINGS 10      /* maximum number of geometry mappings */
+#define MAXCONNECTIONS 100  /* maximum number of connections in dual graph */
+#define MAXBCS 1000         /* maximum number of BCs in naming */
+#define MAXBODIES 100       /* maximum number of bodies in naming */
+#define MAXPARTITIONS 500   /* maximum number of partitions */
+
+#define CONPLAIN 0
+#define CONDISCONT 1
+#define CONPERIODIC 2
+#define CONCONSTRAINT 3
 
 /* Struture GridType includes the subcell structure of the 
    geometry and the meshing information. The elements may be 
@@ -157,10 +166,24 @@ struct FemType {
     variables,     /* number of variables */
     indexwidth,    /* maximum difference of node indices */
     mapgeo,        /* mappings for geometry */
+    *dualgraph[MAXCONNECTIONS],  
+    dualmaxconnections,
+    dualexists,
+
+    *partitiontable[MAXCONNECTIONS],  
+    maxpartitiontable,
+    partitiontableexists, 
+
+    *invtopo[MAXCONNECTIONS],
+    maxinvtopo,
+    invtopoexists,
+
     nocorners,     /* number material corners in the mesh */
     timesteps,     /* number of timesteps */
     periodicexist, /* does the periodic vector exist? */
     *periodic,     /* peridic ordering vector, if needed */
+    connectexist,  /* does the connection vector exist? */
+    *connect,      /* connections between nodes, if needed */
     partitionexist,/* does the partitioning exist? */
     nopartitions,  /* number of partitions */
     *elempart,     /* which partition owns the element */
@@ -170,7 +193,9 @@ struct FemType {
     *material,     /* material for each element */
     **topology,    /* element topology */
     pelems,        /* are there p-type elements */
-    *pelemtypes;
+    *pelemtypes,
+    bodynamesexist,
+    boundarynamesexist;
   int edofs[MAXDOFS],   /* number of dofs in each node */
     eorder[MAXDOFS],    /* does order exist */
     bandwidth[MAXDOFS], /* bandwidth accounting fixed points */
@@ -184,7 +209,8 @@ struct FemType {
       *times;
   Real *dofs[MAXDOFS];  /* degrees of freedom in the mesh */
   char dofname[MAXDOFS][MAXNAMESIZE]; 
-  char materialname[MAXMATERIALS][MAXNAMESIZE];
+  char bodyname[MAXBODIES][MAXNAMESIZE];
+  char boundaryname[MAXBCS][MAXNAMESIZE];
   int noboundaries,              /* number of boundaries */
       boundint[MAXBOUNDARIES],   /* internal material in the boundary */
       boundext[MAXBOUNDARIES],   /* external material in the boundary */
@@ -200,9 +226,6 @@ struct FemType {
    enables the use of a wide variety of BCs. */
 struct BoundaryType {
   int created,       /* is boundary created? */
-#if 0
-    type,            /* type of the boundary */
-#endif
     nosides,         /* sides on the boundary */
     maxsidenodes,  /* number of sidenodes on the element */
     fixedpoints,     /* number of fixed points allowed */
@@ -213,9 +236,7 @@ struct BoundaryType {
     mapvf,           /* mappings of the view factors */ 
     open,            /* is the closure partially open? */
     echain,          /* does the chain exist? */
-#if 0
-    discontinuous;   /* are there secondary nodes? */
-#endif
+    ediscont,        /* does the discontinous boundary exist */
     chainsize;       /* size of the chain */ 
   int *parent,       /* primary parents of the sides */
     *parent2,        /* secondary parents of the sides */
@@ -224,7 +245,7 @@ struct BoundaryType {
     *side2,          /* side in the secondary parent element */
     *chain,          /* indices in the chain representation */
     *types,
-    *discont,        /* are the elements discontinuous (1) or periodic (2) */
+    *discont,        /* type of discontinuous and periodic BCs */
     *normal,         /* direction of the normal */
     points[MAXVARS], /* how many points for each side? */
     evars[MAXVARS];  /* does the variables exist? */
@@ -265,32 +286,64 @@ struct ElmergridType {
   int dim,
     scale,      /* scale the geometry */
     order,      /* reorder the nodes */
-    merge,
-    translate,
-    rotate,
-    clone[3],
-    mirror[3],
-    decimals,
-    layers, 
+    merge,      /* merge mesges */
+    translate,  /* translate the mesh */
+    rotate,     /* rotate the mesh */
+    clone[3],   /* clone the mesh the number of given times */
+    mirror[3],  /* mirror the mash around the given axis */
+    decimals,   /* save the mesh with number of decimals */
+    layers,     /* create boundary layers */
     layerbounds[MAXBOUNDARIES], 
     layernumber[MAXBOUNDARIES], 
-    layermove,
-    metis,
-    partitions, 
+    layermove,  /* map the created layer to the original geometry */
+    metis,      /* number of Metis partitions */
+    metisopt,
+    partitions, /* number of simple geometric partitions */
     partdim[3],
-    inmethod,
-    outmethod,
+    inmethod,   /* method in which mesh is read in to ElmerGrid */
+    outmethod,  /* method in which the mesh is written by ElmerGrid */
     sidemap[3*MAXBOUNDARIES],
     sidemappings,
     bulkmap[3*MAXMATERIALS],
     bulkmappings,
+    boundorder, 
+    bulkorder, 
     boundbounds,
     boundbound[3*MAXBOUNDARIES],
     bulkbounds,
-    bulkbound[3*MAXBOUNDARIES];
-      
-  int mirrorbc;
-  
+    bulkbound[3*MAXBOUNDARIES], 
+    mirrorbc,
+    layerparents[MAXBOUNDARIES],
+    sidebulk[MAXSIDEBULK],
+    triangles,
+    polar,
+    usenames, 
+    isoparam,
+    cylinder,
+    unitemeshes,
+    reduce,
+    removelowdim,
+    increase,
+    reducemat1,
+    reducemat2,
+    findsides,
+    saveboundaries,
+    nodes3d,
+    elements3d,
+    periodic, 
+    periodicdim[3],
+    discont,
+    discontbounds[MAXBOUNDARIES],
+    connect,
+    connectbounds[MAXBOUNDARIES],
+    partorder,
+    nofilesin,
+    elementsredone,
+    pelemmap[4*MAXMATERIALS],pelems,
+    belemmap[4*MAXMATERIALS], belems,
+    advancedelem[7*MAXMATERIALS], advancedmat,
+    bcoffset;
+
   Real cscale[3], 
     corder[3],
     cmerge,
@@ -299,21 +352,12 @@ struct ElmergridType {
     clonesize[3],
     layerratios[MAXBOUNDARIES], 
     layerthickness[MAXBOUNDARIES],
-    layereps;
+    layereps,
+    partcorder[3],
+    polarradius;
 
-  int layerparents[MAXBOUNDARIES];
-  int sidebulk[MAXSIDEBULK];
-  int triangles,polar,isoparam,cylinder;
-  int unitemeshes,reduce,increase,reducemat1,reducemat2;
-  int findsides,saveboundaries,nodes3d,elements3d;
-  int periodic, periodicdim[3],discont,discontbounds[MAXBOUNDARIES];
-  int partorder,pelemmap[4*MAXMATERIALS],pelems,nofilesin,elementsredone;
-  int belemmap[4*MAXMATERIALS], belems;
-  int advancedelem[7*MAXMATERIALS], advancedmat;
-  int bcoffset;
-
-  Real partcorder[3],polarradius;
-  char filesin[MAXCASES][MAXFILESIZE],filesout[MAXCASES][MAXFILESIZE];
-  char mapfile[MAXFILESIZE];
+  char filesin[MAXCASES][MAXFILESIZE],
+    filesout[MAXCASES][MAXFILESIZE], 
+    mapfile[MAXFILESIZE];
 };
 
